#pragma once

#include <utility>
#include <type_traits>

#include <dune/common/concept.hh>

#include <dune/functions/functionspacebases/concepts.hh>
#include <dune/functions/common/indexaccess.hh>

#include <amdis/common/ConceptsBase.hpp>

namespace AMDiS
{
  /**
  * \brief A simple node to range map using the nested tree indices
  *
  * This map directly usses the tree path entries of the given
  * node to access the nested container.
  *
  * If the container does not provide any operator[] access,
  * it is simply forwarded for all nodes.
  */
  struct HierarchicNodeToRangeMap
  {
    template <class Node, class TreePath, class Range,
      REQUIRES(Dune::models<Dune::Functions::Concept::HasIndexAccess, Range, Dune::index_constant<0>>())>
    decltype(auto) operator()(const Node& node, const TreePath& treePath, Range&& y) const
    {
      return Dune::Functions::resolveStaticMultiIndex(y, treePath);
    }

    template <class Node, class TreePath, class Range,
      REQUIRES(not Dune::models<Dune::Functions::Concept::HasIndexAccess, Range, Dune::index_constant<0>>())>
    decltype(auto) operator()(const Node& node, const TreePath& treePath, Range&& y) const
    {
      return std::forward<Range>(y);
    }
  };

} // end namespace AMDiS
